#!/usr/bin/perl
#
################################################################################
#
# IPCop WakeOnLan-AddOn CGI
#
#         losely based on IPCop dhcp.cgi, hosts.cgi & wireless.cgi 
#
# Copyright (C) 2006-2014 Olaf (weizen_42) Westrik
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################
#
# For support post / read in http://www.ipcop-forum.de
#
# $Id: wakeonlan-gui.cgi 532 2014-04-09 08:24:34Z weizen_42 $
#
# Revision history:
#
# 2006-03-xx created by weizen_42 for wolgui-addon
#
# 2006-03-05  weizen_42       major surgery
# 2006-03-07  weizen_42       v0.2.0  added edit functionality
#                                     almost complete rewrite of add/edit/update strategy
# 2006-03-07  weizen_42       v0.3.0  show edited device with yellow table background
#                                     remove wakeup-button if IFace not available
# 2006-09-21  weizen_42       v0.4.0  some cosmetics
# 2007-03-22  weizen_42       v0.5.0  allow MAC on multiple interfaces, thanks to dotzball
#                                     new version detection
#

# Add entry in menu
# MENUENTRY network 091 "WakeOnLan" "WakeOnLan"

use strict;
use LWP::UserAgent;

# enable only the following on debugging purpose
use warnings;
use CGI::Carp 'fatalsToBrowser';

require '/usr/lib/ipcop/general-functions.pl';
require '/usr/lib/ipcop/lang.pl';
require '/usr/lib/ipcop/header.pl';

my $version = 'v1.0.2';
# checking for new version
my $addonname = 'wakeonlan';
my $onlineversion = '';
my $onlinelink = '';
my $timestamplastcheck = '/var/ipcop/addons/${addonname}/lastcheck';
my $flagdonotcheck = '/var/ipcop/addons/${addonname}/noversioncheck';


# remove comment from next line to get wakeup info in seperate page
my $refresh = 'yes';
# remove comment from next line to get wakeup info as inline box
#my $refresh = '';


#workaround to suppress a warning when a variable is used only once
#my @dummy = ( ${Header::colouryellow} );
#undef (@dummy);
my $line;
my $i;
my $ic;

my @wol_devices = ();
my %netsettings = ();
&General::readhash('/var/ipcop/ethernet/settings', \%netsettings);
my %cgiparams = ();
#configfile
our $datafile = "/var/ipcop/addons/wakeonlan-gui/clients.conf";
&ReadConfig;


&General::getcgihash(\%cgiparams);

my @INTERFACEs = ('GREEN', 'BLUE', 'ORANGE');
my %selected = ();
my $interface;
foreach $interface (@INTERFACEs) {
    $i = $netsettings{"${interface}_COUNT"};
    while ($i > 0) {
        $selected{'CLIENT_IFACE'}{"${interface}_${i}"} = '';
        $i--;
    }
}

&Header::showhttpheaders();

my $errormessage = "";

if ($cgiparams{'ACTION'} eq 'add') {
    # add a device, check for valid and non-duplicate MAC
    if ($cgiparams{'CLIENT_MAC'} eq '') {
        goto ADDEXIT;
    }

    $cgiparams{'CLIENT_MAC'} =~ tr/-/:/;

    unless (&General::validmac($cgiparams{'CLIENT_MAC'})) {
        $errormessage = $Lang::tr{'invalid mac address'}; 
        goto ADDEXIT;
    }

    for $i (0 .. $#wol_devices) {
        if (lc($cgiparams{'CLIENT_MAC'}) eq lc($wol_devices[$i]{'MAC'}) && $cgiparams{'CLIENT_IFACE'} eq $wol_devices[$i]{'IFace'}) {
            $errormessage = $Lang::tr{'duplicate mac'};
            goto ADDEXIT;
        }
    }

    unless ($errormessage) {
        push @wol_devices, { MAC => uc($cgiparams{'CLIENT_MAC'}), IFace => $cgiparams{'CLIENT_IFACE'}, Comment => $cgiparams{'CLIENT_COMMENT'} };
        &WriteConfig;
    }

ADDEXIT:
# jump here to keep cgiparams!
}

if ($cgiparams{'ACTION'} eq 'update') {
    # update a device, check for valid and non-duplicate MAC
    if ($cgiparams{'CLIENT_MAC'} eq '') {
        goto UPDATEEXIT;
    }

    $cgiparams{'CLIENT_MAC'} =~ tr/-/:/;

    unless (&General::validmac($cgiparams{'CLIENT_MAC'})) {
        $errormessage = $Lang::tr{'invalid mac address'}; 
        goto UPDATEEXIT;
    }

    for $i (0 .. $#wol_devices) {
        next if ($i == $cgiparams{'ID'});
        if (lc($cgiparams{'CLIENT_MAC'}) eq lc($wol_devices[$i]{'MAC'}) && $cgiparams{'CLIENT_IFACE'} eq $wol_devices[$i]{'IFace'}) {
            $errormessage = $Lang::tr{'duplicate mac'};
            goto UPDATEEXIT;
        }
    }

    unless ($errormessage) {
        $wol_devices[$cgiparams{'ID'}]{'MAC'} = $cgiparams{'CLIENT_MAC'};
        $wol_devices[$cgiparams{'ID'}]{'IFace'} = $cgiparams{'CLIENT_IFACE'};
        $wol_devices[$cgiparams{'ID'}]{'Comment'} = $cgiparams{'CLIENT_COMMENT'};
        &WriteConfig;
    }

UPDATEEXIT:
# jump here to keep cgiparams!
}

if ($cgiparams{'ACTION'} eq 'remove') {
    # simply set MAC to empty, WriteConfig will handle the gory details
    $wol_devices[$cgiparams{'ID'}]{'MAC'} = '';
    &WriteConfig;
}

if (($cgiparams{'ACTION'} ne 'wakeup') || ($refresh ne 'yes')) {
    &Header::openpage($Lang::tr{'WakeOnLan'}, 1, '');
    &Header::openbigbox('100%', 'left', '', $errormessage);
}

if ($cgiparams{'ACTION'} eq 'wakeup') {
    # wakey wakey
    my $mac = $wol_devices[$cgiparams{'ID'}]{'MAC'};
    my $iface = uc($wol_devices[$cgiparams{'ID'}]{'IFace'}).'_DEV';
    $iface = $netsettings{"$iface"};
    my $wol_txt = &Header::cleanhtml($wol_devices[$cgiparams{'ID'}]{'Comment'});

    undef %cgiparams;

    system("/usr/sbin/etherwake -i $iface $mac");

    # make a box with info, 'refresh' to normal screen after 5 seconds
    if ($refresh eq 'yes') {
        &Header::openpage($Lang::tr{'WakeOnLan'}, 1, "<meta http-equiv='refresh' content='3;url=/cgi-bin/wakeonlan-gui.cgi' />");
        &Header::openbigbox('100%', 'left');
    }
    &Header::openbox('100%', 'left', $Lang::tr{'WakeOnLan'});

    if ($wol_txt ne '') {
        print "<p>$Lang::tr{'magic packet send to:'} $wol_txt ($iface, $mac)</p>";
    }
    else {
        print "<p>$Lang::tr{'magic packet send to:'} $mac ($iface)</p>";  
    }
  
    &Header::closebox();

    if ($refresh eq 'yes') {
        &Header::closebigbox();
        &Header::closepage();
        # that's all folks
        exit;
    }
}

#print "Action: $cgiparams{'ACTION'}<br />";
#print "ID: $cgiparams{'ID'}<br />";
#print "MAC: $cgiparams{'CLIENT_MAC'}<br />";
#print "IFace: $cgiparams{'CLIENT_IFACE'}<br />";
#print "Rem: $cgiparams{'CLIENT_COMMENT'}<br />";

if ($errormessage) {
    # some error from add / update
    &Header::openbox('100%', 'left', $Lang::tr{'error messages'}, 'error');
    print "<font class='base'>$errormessage</font>";
    &Header::closebox();
}


#
# Check for new version
#
&checkfornewversion($addonname, $version);
if ($onlineversion ne '') {
    &Header::openbox('100%', 'left', $Lang::tr{'info'}, 'warning');
    print <<END
<table width="100%"><tr>
<td>$Lang::tr{'wol newversion'} <a href="$onlinelink" target="_blank"><b>$onlineversion</b></a></td>
</tr></table>
END
;
    &Header::closebox();
}


print "<form method='post' action='$ENV{'SCRIPT_NAME'}'>\n";

$selected{'CLIENT_IFACE'}{$cgiparams{'CLIENT_IFACE'}} = "selected='selected'";
my $buttontext = $Lang::tr{'add'};
if ($cgiparams{'ACTION'} eq 'edit') {
    &Header::openbox('100%', 'left', "$Lang::tr{'edit device'}");
    $buttontext = $Lang::tr{'update'};
    $cgiparams{'CLIENT_MAC'} = $wol_devices[$cgiparams{'ID'}]{'MAC'};
    $selected{'CLIENT_IFACE'}{$wol_devices[$cgiparams{'ID'}]{'IFace'}} = "selected='selected'";
    $cgiparams{'CLIENT_COMMENT'} = $wol_devices[$cgiparams{'ID'}]{'Comment'};
}
elsif ($cgiparams{'ACTION'} eq 'update') {
    &Header::openbox('100%', 'left', "$Lang::tr{'edit device'}");
    $buttontext = $Lang::tr{'update'};
}
else {
    &Header::openbox('100%', 'left', "$Lang::tr{'add device'}");
}

print <<END
<table width='100%'>
<tr>
    <td width='15%' class='base'>$Lang::tr{'mac address'}:&nbsp;</td>
    <td width='40%'><input type='text' name='CLIENT_MAC' value='$cgiparams{'CLIENT_MAC'}' size='25' /></td>
    <td width='10%' class='base'>$Lang::tr{'interface'}:&nbsp;</td>
    <td align='left'>
        <select name='CLIENT_IFACE'>
END
;

foreach $interface (@INTERFACEs) {
    for ($i = 1; $i <= $netsettings{"${interface}_COUNT"}; $i++) {
        $ic = "${interface}_${i}";
        my $lc_interface = $Lang::tr{lc($interface)};
        print "<option value='${ic}' $selected{'CLIENT_IFACE'}{$ic}>$lc_interface $i</option>";
    }
}

print <<END
    </select>
  </td>
</tr>
<tr>
    <td width='15%' class='base'>$Lang::tr{'remark'}:&nbsp;<img src='/blob.gif' alt='*' /></td>
    <td colspan='4' align='left'><input type='text' name='CLIENT_COMMENT' value='$cgiparams{'CLIENT_COMMENT'}' size='40' /></td>
</tr>
</table>
<hr />
<table width='100%'>
<tr>
    <td class='comment1button'><img src='/blob.gif' alt='*' />&nbsp;$Lang::tr{'this field may be blank'}</td>
    <td class='button1button'>
END
;

if (($cgiparams{'ACTION'} eq 'edit') || ($cgiparams{'ACTION'} eq 'update')) {
    print "<input type='hidden' name='ID' value='$cgiparams{'ID'}' />\n";
    print "<input type='hidden' name='ACTION' value='update' />";
}
else {
    print "<input type='hidden' name='ACTION' value='add' />";
}
print <<END
        <input type='submit' name='SUBMIT' value='$buttontext' />
    </td>
    <td class='onlinehelp'>
        <a href='http://www.ban-solms.de/t/IPCop-wolgui.html' target='_blank'>
        <img src='/images/web-support.png' alt='Online help' title='Online help' /></a>
    </td>
</tr>
</table>
END
;

&Header::closebox();

print "</form>\n";

#######################################
#
# now list already configured devivces
#
#######################################
&Header::openbox('100%', 'left', "$Lang::tr{'current devices'}");

print <<END
<table width='100%'>
<tr>
<td align='center' width='20%'><b>$Lang::tr{'mac address'}</b></td>
<td align='center' width='10%'><b>$Lang::tr{'interface'}</b></td>
<td align='center' width='60%'><b>$Lang::tr{'remark'}</b></td>
<td align='center' colspan='2'><b>$Lang::tr{'action'}</b></td>
</tr>
END
;

for $i (0 .. $#wol_devices) {
    my $wol_mac = $wol_devices[$i]{'MAC'};
    my $wol_iface = $wol_devices[$i]{'IFace'};
    my $wol_txt = &Header::cleanhtml($wol_devices[$i]{'Comment'});

    if ((($cgiparams{'ACTION'} eq 'edit') || ($cgiparams{'ACTION'} eq 'update')) && ($i == $cgiparams{'ID'})) {
        print "<tr class='selectcolour'>";
    }
    else {
        print "<tr class='table".int(($i % 2) + 1)."colour'>";
    }

    my @lc_interface = split(/_/, lc($wol_devices[$i]{'IFace'}), 2);
    print <<END
<td align='center'>$wol_mac</td>
<td align='center'>$Lang::tr{"$lc_interface[0]"} $lc_interface[1]</td>
<td align='left'>$wol_txt</td>
<td align='center'>
END
;
#    if ((($wol_iface eq 'blue') && ! &haveBlueNet()) || (($wol_iface eq 'orange') && ! &haveOrangeNet())) {
#        # configured IFace (momentarily) not available -> no wakeup button/image
#        print "&nbsp;";
#    }
#    else {
        print <<END
<form method='post' name='frma$i' action='$ENV{'SCRIPT_NAME'}'>
<input type='hidden' name='ACTION' value='wakeup' />
<input type='image' name='wakeup' src='/images/wakeup.gif' alt='$Lang::tr{'wol wakeup'}' title='$Lang::tr{'wol wakeup'}' />
<input type='hidden' name='ID' value='$i' />
</form>
END
        ;
#    }
    print <<END
</td>
<td align='center'>
    <form method='post' name='frmb$i' action='$ENV{'SCRIPT_NAME'}'>
    <input type='hidden' name='ACTION' value='edit' />
    <input type='image' name='$Lang::tr{'edit'}' src='/images/edit.gif' alt='$Lang::tr{'edit'}' title='$Lang::tr{'edit'}' />
    <input type='hidden' name='ID' value='$i' />
    </form>
</td>
<td align='center'>
    <form method='post' name='frmc$i' action='$ENV{'SCRIPT_NAME'}'>
    <input type='hidden' name='ACTION' value='remove' />
    <input type='image' name='$Lang::tr{'remove'}' src='/images/delete.gif' alt='$Lang::tr{'remove'}' title='$Lang::tr{'remove'}' />
    <input type='hidden' name='ID' value='$i' />
    </form>
</td>
END
    ;
    print "</tr>\n";
}

print <<END
</table>

<br />
<hr />
<table width='100%'>
<tr>
    <td>&nbsp;</td>
    <td align='right'>
    <b><small><a href="http://www.ban-solms.de/t/IPCop.html" target="_blank">Wake-on-LAN GUI $version</a></small></b>
    </td>
</tr>
</table>
END
;


&Header::closebox();

&Header::closebigbox();
&Header::closepage();


#
# load the configuration file
#
sub ReadConfig
{
    # datafileformat:
    #   ID,MAC,IFACE,,Comment
    # 
    my @tmpfile = ();
    if (open(FILE, "$datafile")) {
        @tmpfile = <FILE>;
        close (FILE);
    }

    @wol_devices = ();

    # populate devices list
    foreach $line (@tmpfile) {
        chomp($line);               # remove newline
        my @temp = split(/\,/,$line,5);
        next if ($temp[1] eq '');
        next unless (&General::validmac($temp[1]));

        push @wol_devices, { ID => $temp[0], MAC => $temp[1], IFace => $temp[2], Comment => $temp[4] };
    }

    undef %cgiparams;
    $cgiparams{'ACTION'} = '';    # add/edit/update/remove/wakeup
    $cgiparams{'ID'} = '';        # points to record for ACTION (edit/update/remove)
    $cgiparams{'CLIENT_MAC'} = '';
    $cgiparams{'CLIENT_IFACE'} = '';
    $cgiparams{'CLIENT_COMMENT'} = '';
}

#
# write the configuration file
#
sub WriteConfig
{
    my $line;
    my @temp;

    my @tmp_clients;

    for $i (0 .. $#wol_devices) {
        next unless(&General::validmac($wol_devices[$i]{'MAC'}));
        unshift (@tmp_clients, uc($wol_devices[$i]{'MAC'}).",$wol_devices[$i]{'IFace'},,$wol_devices[$i]{'Comment'}");
    }

    # sort tmp_clients on MAC
    @tmp_clients = sort ( @tmp_clients );

    open(FILE, ">$datafile") or die 'hosts datafile error';

    my $count = 0;
    foreach $line (@tmp_clients) {
        print FILE "$count,$line\n";
        $count++;
    }
    close FILE;

    &ReadConfig;
}


sub checkfornewversion
{
    my $addon = shift;
    my $version = shift;
    $onlineversion = '';

    if ( -e $flagdonotcheck ) {
        return;
    }

    # only check if we are online and last check was some time ago
    if ( (! -e '/var/ipcop/red/active') || (-e $timestamplastcheck) && (int(-M $timestamplastcheck) < 5) ) {
        return;
    }

    #workaround to suppress a warning when a variable is used only once
    my @dummy = ( $General::version );
    undef (@dummy);

    my $ua = LWP::UserAgent->new;
    $ua->timeout(120);
    $ua->agent("Mozilla/4.0 (compatible; IPCop $General::version; $version)");
    my $content = $ua->get("http://www.ban-solms.de/version/$addon");

    if ( $content->is_success ) {
        # compare the versions, format is v1.2.3
        $content->content =~ /v(\d+).(\d+).(\d+)/;
        my $ver1 = $1;
        my $ver2 = $2;
        my $ver3 = $3;

        $version =~ /v(\d+).(\d+).(\d+)/;

        if ( ($ver1 > $1) || (($ver1 == $1) && ($ver2 > $2)) || (($ver1 == $1) && ($ver2 == $2) && ($ver3 > $3)) ) {
            $onlineversion = "v$ver1.$ver2.$ver3";

            $content->content =~ /http(.*)/;
            $onlinelink = "http$1";
        }
        else {
            # no news, recheck in a couple of days
            system("touch $timestamplastcheck");
        }
    }
}
